/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <openssl/des.h>
// To compile on Windows you must download OpenSSL from here:
//   http://www.slproweb.com/products/Win32OpenSSL.html



#define VER         "0.1"
#define LPINFOKEY   "\xe3\x25\xbf\x25\x4f\xcd\x4c\xbf"  // info.dat
#define LPUDPKEY    "\x8c\xe0\x67\x5d\xd9\xf8\xea\xf1"  // packet



void std_err(char *error);



int main(int argc, char *argv[]) {
    FILE                *fd;
    int                 i,
                        len;
    unsigned char       *buff,
                        *ptr;
    struct  stat        xstat;
    DES_key_schedule    ks;
    DES_cblock          ivec;


    setbuf(stdout, NULL);

    fputs("\n"
        "Station Launchpad profile and packet password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <info.dat_or_packetdump>\n"
            "\n"
            "Input file:\n"
            "  with this tool you can decrypt the encoded password stored in the profile\n"
            "  file called info.dat or that stored in the UDP packet that your Launchpad\n"
            "  client sends to \"sdlaunchpad1.station.sony.com\" on port 3016, 3017 or 9700.\n"
            "  In this second case you need to dump the data of the packet (you easily see\n"
            "  it because it contains the username in plain-text) in a file.\n"
            "\n"
            "  info.dat is a profile file located in the directory Users\\NICKNAME where\n"
            "  NICKNAME is the account about it is referred.\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("- open file \"%s\"\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err(NULL);

    fstat(fileno(fd), &xstat);
    len = xstat.st_size - 5;
    buff = malloc(len);
    if(!buff) std_err(NULL);

    fread(buff, 5, 1, fd);
    if(!*buff) {    // packet dump

        fputs("- recognized as packet dump\n", stdout);

        if(fseek(fd, 15, SEEK_CUR) < 0) std_err(NULL);
        len -= 15;
        fread(buff, len, 1, fd);
        fclose(fd);

        printf("\n Username:   %s\n", buff);
        ptr = buff + strlen(buff) + 4 + 1;
        len -= (ptr - buff);

        ptr[len] = 0;
        memcpy(ivec, LPUDPKEY, 8);

    } else {        // info.dat

        fputs("- recognized as account file\n", stdout);

        if(memcmp(buff, "true ", 5)) {
            std_err("no password has been stored in this account file");
        }

        fread(buff, len, 1, fd);
        fclose(fd);

        ptr = strchr(buff, 0x01);
        if(!ptr) {
            std_err("the password has not been found");
        }
        *ptr = 0;
        printf("\n Username:   %s\n", buff);

        len = (len - (ptr - buff)) >> 1;
        for(ptr++, i = 0; i < len; i++) {
            ptr[i] = ptr[i << 1];   // skip 0x01
        }
        ptr[len] = 0;

        memcpy(ivec, LPINFOKEY, 8);
    }

    DES_set_key(&ivec, &ks);
    DES_cbc_encrypt(ptr, ptr, len, &ks, &ivec, 0);
    printf(" Password:   %s\n", ptr);

    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void std_err(char *error) {
    if(!error) perror("\nError");
        else printf("\nError: %s\n\n", error);
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}

